<?php
/**
 * Copyright (c) 2024
 * @author: Alttantire Yazılım Çözümleri <info@alttantire.com>
 * Web: https://www.alttantire.com
 *
 */

namespace Alttantire\HepsiPaySDK;

use Alttantire\Client;
use Alttantire\HandlerStack;
use Alttantire\Middleware;


/**
 *
 */
class HepsiPayGateway
{

    /**
     * @var string
     */
    protected string $url = "";
    /**
     * @var string
     */
    protected string $endpoint;
    /**
     * @var string
     */
    protected string $merchant_no;
    /**
     * @var string
     */
    protected string $terminal_no;
    /**
     * @var string
     */
    protected string $store_key;
    /**
     * @var string
     */
    protected string $communication_method = "cURL";
    /**
     * @var int
     */
    private int $frame_type;
    /**
     * @var int
     */
    private int $amount;
    /**
     * @var string
     */
    private string $account_key;
    /**
     * @var string
     */
    private string $merchant_callback_url;
    /**
     * @var string
     */
    private string $merchant_order_number = "";
    /**
     * @var string
     */
    private string $checkout_merchant_url = "";
    /**
     * @var string
     */
    private string $bank_order_id = "";
    /**
     * @var string
     */
    private string $external_id = "";
    /**
     * @var string
     */
    private string $pos_alias;
    /**
     * @var array
     */
    private array $basket_items;
    /**
     * @var array
     */
    private array $allowed_installments = [];
    /**
     * @var bool
     */
    private bool $is_guest_user = false;
    /**
     * @var bool
     */
    private bool $force_3ds = false;
    /**
     * @var int
     */
    private int $refund_amount;
    /**
     * @var string
     */
    private string $client_ip;
    /**
     * @var string
     */
    private string $payment_id;
    /**
     * @var string
     */
    private string $reference_no;

    /**
     * @param $endpoint
     * @param $merchant_no
     * @param $terminal_no
     * @param $store_key
     */
    public function __construct($endpoint, $merchant_no, $terminal_no, $store_key)
    {
        $this->endpoint = $endpoint;
        $this->merchant_no = $merchant_no;
        $this->terminal_no = $terminal_no;
        $this->store_key = $store_key;

        $this->init();
    }

    /**
     * @return $this
     */
    public function init(): static
    {
        function ends_with($haystack, $needle, $case = true): bool
        {
            if ($case) {
                return (strcmp(substr($haystack, strlen($haystack) - strlen($needle)), $needle) === 0);
            }

            return (strcasecmp(substr($haystack, strlen($haystack) - strlen($needle)), $needle) === 0);
        }


        if (!ends_with($this->endpoint, "/")) {
            $this->endpoint = $this->endpoint . "/";
        }

        $this->detect_curl_availability();

        return $this;
    }


    /**
     * @param int $amount
     * @return void
     */
    public function set_amount(int $amount): void
    {
        $this->amount = $amount;
    }

    /**
     * @param string $account_key
     * @return void
     */
    public function set_account_key(string $account_key): void
    {
        $this->account_key = $account_key;
    }

    /**
     * @param string $merchant_callback_url
     * @return void
     */
    public function set_merchant_callback_url(string $merchant_callback_url): void
    {
        $this->merchant_callback_url = $merchant_callback_url;
    }

    /**
     * @param array $basket_item
     * @return void
     */
    public function set_basket_item(array $basket_item): void
    {
        $this->basket_items[] = json_decode(json_encode($basket_item));
    }

    /**
     * @param bool $is_guest_user
     * @return void
     */
    public function set_is_guest_user(bool $is_guest_user): void
    {
        $this->is_guest_user = $is_guest_user;
    }

    /**
     * @param bool $force_3ds
     * @return void
     */
    public function set_force_3ds(bool $force_3ds): void
    {
        $this->force_3ds = $force_3ds;
    }

    /**
     * @param array $allowed_installments
     * @return void
     */
    public function set_allowed_installments(array $allowed_installments): void
    {
        $this->allowed_installments = $allowed_installments;
    }

    /**
     * @param string $merchant_order_number
     * @return void
     */
    public function set_merchant_order_number(string $merchant_order_number): void
    {
        $this->merchant_order_number = $merchant_order_number;
    }

    /**
     * @param string $bank_order_id
     * @return void
     */
    public function set_bank_order_id(string $bank_order_id): void
    {
        $this->bank_order_id = $bank_order_id;
    }

    /**
     * @param string $external_id
     * @return void
     */
    public function set_external_id(string $external_id): void
    {
        $this->external_id = $external_id;
    }

    /**
     * @param int $frame_type
     * @return void
     */
    public function set_frame_type(int $frame_type): void
    {
        $this->frame_type = $frame_type;
    }

    /**
     * @param string $checkout_merchant_url
     * @return void
     */
    public function set_checkout_merchant_url(string $checkout_merchant_url): void
    {
        $this->checkout_merchant_url = $checkout_merchant_url;
    }

    /**
     * @param string $pos_alias
     * @return void
     */
    public function set_pos_alias(string $pos_alias): void
    {
        $this->pos_alias = $pos_alias;
    }

    /**
     * @param int $refund_amount
     * @return void
     */
    public function set_refund_amount(int $refund_amount): void
    {
        $this->refund_amount = $refund_amount;
    }

    /**
     * @param string $client_ip
     * @return void
     */
    public function set_client_ip(string $client_ip): void
    {
        $this->client_ip = $client_ip;
    }

    /**
     * @param string $payment_id
     * @return void
     */
    public function set_payment_id(string $payment_id): void
    {
        $this->payment_id = $payment_id;
    }

    /**
     * @param string $reference_no
     * @return void
     */
    public function set_reference_no(string $reference_no): void
    {
        $this->reference_no = $reference_no;
    }

    /**
     * @return string
     */
    public function get_request_body_string(): string
    {

        $body = [];

        if (!empty($this->amount)) {
            $body["Amount"] = $this->amount;
        }

        if (!empty($this->account_key)) {
            $body["AccountKey"] = $this->account_key;
        }

        if (!empty($this->merchant_callback_url)) {
            $body["MerchantCallBackUrl"] = $this->merchant_callback_url;
        }

        if (!empty($this->basket_items)) {
            $body["BasketItems"] = $this->basket_items;
        }

        if (!empty($this->is_guest_user)) {
            $body["IsGuestUser"] = $this->is_guest_user;
        }

        if (!empty($this->force_3ds)) {
            $body["Force3ds"] = $this->force_3ds;
        }

        if (!empty($this->allowed_installments)) {
            $body["AllowedInstallments"] = $this->allowed_installments;
        }

        if (!empty($this->frame_type)) {
            $body["FrameType"] = $this->frame_type;
        }

        if (!empty($this->merchant_order_number)) {
            $body["MerchantOrderNumber"] = $this->merchant_order_number;
        }

        if (!empty($this->bank_order_id)) {
            $body["BankOrderId"] = $this->bank_order_id;
        }

        if (!empty($this->external_id)) {
            $body["ExternalId"] = $this->external_id;
        }

        if (isset($this->frame_type) && $this->frame_type === 2) {
            $body["CheckoutMerchantUrl"] = $this->checkout_merchant_url;
        }

        if (!empty($this->pos_alias)) {
            $body["PosAlias"] = $this->pos_alias;
        }

        if (!empty($this->refund_amount)) {
            $body["refundAmount"] = $this->refund_amount;
        }

        if (!empty($this->client_ip)) {
            $body["clientIp"] = $this->client_ip;
        }

        if (!empty($this->payment_id)) {
            $body["paymentId"] = $this->payment_id;
        }

        if (!empty($this->reference_no)) {
            $body["referenceNo"] = $this->reference_no;
        }

        if (count($body) === 0) {
            return "";
        }

        return json_encode($body);

    }

    /**
     * @return void
     */
    public function detect_curl_availability(): void
    {
        if (function_exists('curl_init')) {
            $this->communication_method = "cURL";
            return;
        }

        if (function_exists('file_get_contents') && ini_get('allow_url_fopen')) {
            $this->communication_method = "fopen";
            return;
        }
    }

    /**
     * @throws HepsiPaySDKException
     */
    public function post($url, $header_params = [])
    {

        return match ($this->communication_method) {
            "cURL" => $this->post_with_curl($url, $header_params),
            "fopen" => $this->post_with_file_contents($url, $header_params),
            default => false,
        };
    }

    /**
     * @throws HepsiPaySDKException
     */
    public function get($url, $header_params = [])
    {
        return match ($this->communication_method) {
            "cURL" => $this->get_with_curl($url, $header_params),
            "fopen" => $this->get_with_file_contents($url, $header_params),
            default => false,
        };
    }

    /**
     * @throws HepsiPaySDKException
     * @throws \Exception
     */
    function post_with_file_contents($path, $header_params = [])
    {

        $path = $this->endpoint . $path;

        $header = ["Content-Type: application/json"];
        foreach ($header_params as $key => $value) {
            $header[] = $key . ": " . $value;
        }

        $data_string = ($this->get_request_body_string());

        $opts = array(
            'http' =>
                array(
                    'method' => 'POST',
                    'header' => implode("\r\n", $header),
                    'timeout' => 10,
                    'content' => $data_string
                )
        );

        $context = stream_context_create($opts);

        try {

            $fp = @fopen($path, 'r', false, $context);

            if (!$fp) {
                throw new HepsiPaySDKException($http_response_header[0]);
            }

            $response = stream_get_contents($fp);

            $meta_data = stream_get_meta_data($fp);
            $response_headers = $meta_data['wrapper_data'];

            fclose($fp);

            $http_code = null;
            foreach ($response_headers as $header) {
                if (preg_match('#HTTP/\d+\.\d+ (\d+)#', $header, $matches)) {
                    $http_code = intval($matches[1]);
                    break;
                }
            }

            if ($http_code === 401) {
                throw new HepsiPaySDKException("Yetkisiz Erişim. " . $http_code);
            }

            return json_decode((string)$response);


        } catch (HepsiPaySDKException $e) {
            throw new HepsiPaySDKException("İstisna yakalandı: " . $e->getMessage());
        }

    }

    /**
     * @throws HepsiPaySDKException
     * @throws \Exception
     */
    function get_with_file_contents($path, $header_params = [])
    {

        $header = ["Content-Type: application/json"];
        foreach ($header_params as $key => $value) {
            $header[] = $key . ": " . $value;
        }

        try {

            $url = $this->endpoint . $path;

            $options = array(
                'http' => array(
                    'header' => implode("\r\n", $header),
                    'method' => 'GET',
                )
            );

            $context = stream_context_create($options);

            $response = file_get_contents($url, false, $context);

            if ($response === FALSE) {
                throw new HepsiPaySDKException("Hata oluştu ");
            }

            return json_decode((string)$response);

        } catch (HepsiPaySDKException $e) {
            throw new HepsiPaySDKException("İstisna yakalandı: " . $e->getMessage());
        }

    }

    /**
     * @throws Exception
     * @throws HepsiPaySDKException
     */
    public function post_with_curl($url, $header_params = [])
    {
        try {

            $header = ["Content-Type: application/json"];
            foreach ($header_params as $key => $value) {
                $header[] = $key . ": " . $value;
            }

            $data_string = ($this->get_request_body_string());

            $ch = @curl_init($this->endpoint . $url);

            @curl_setopt($ch, CURLOPT_POST, true);
            @curl_setopt($ch, CURLOPT_POSTFIELDS, $data_string);
            @curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
            @curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            @curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 5);
            @curl_setopt($ch, CURLOPT_TIMEOUT, 10); //timeout in seconds

            $result = curl_exec($ch);

            if (@curl_errno($ch)) {
                $error_msg = curl_error($ch);
                @curl_close($ch);
                throw new Exception($error_msg);
            } else {


                $http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
                $header_size = curl_getinfo($ch, CURLINFO_HEADER_SIZE);
                $header = substr($result, 0, $header_size);
                $body = substr($result, $header_size);

                if ($http_code === 401) {
                    throw new HepsiPaySDKException("Yetkisiz Erişim. " . $http_code);
                }

                $data = json_decode($result);
                @curl_close($ch);

                return $data;

            }

        } catch (HepsiPaySDKException $e) {

            throw new HepsiPaySDKException("İstisna yakalandı: " . $e->getMessage());

        } catch (Exception $e) {

            throw new Exception($e->getMessage());
        }

    }

    /**
     * @throws Exception
     * @throws HepsiPaySDKException
     */
    public function get_with_curl($url, $header_params = [])
    {

        try {

            $header = ["Content-Type: application/json"];
            foreach ($header_params as $key => $value) {
                $header[] = $key . ": " . $value;
            }


            $ch = @curl_init($this->endpoint . $url);

//            @curl_setopt($ch, CURLOPT_POST, true);
            @curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
            @curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            @curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 5);
            @curl_setopt($ch, CURLOPT_TIMEOUT, 10); //timeout in seconds

            $result = curl_exec($ch);

            print_r($this->endpoint . $url);exit;

            if (@curl_errno($ch)) {
                $error_msg = curl_error($ch);
                @curl_close($ch);
                throw new Exception($error_msg);
            } else {

                $http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
                $header_size = curl_getinfo($ch, CURLINFO_HEADER_SIZE);
                $header = substr($result, 0, $header_size);
                $body = substr($result, $header_size);

                if ($http_code === 401) {
                    throw new HepsiPaySDKException("Yetkisiz Erişim. " . $http_code);
                }

                $data = json_decode($result);
                @curl_close($ch);

                return $data;

            }

        } catch (HepsiPaySDKException $e) {

            throw new HepsiPaySDKException("İstisna yakalandı: " . $e->getMessage());

        } catch (Exception $e) {

            throw new Exception($e->getMessage());
        }

    }

    /**
     * @param array $params
     * @return array
     */
    public function params(array $params = []): array
    {
        return [
                'merchant-no' => $this->merchant_no,
                'terminal-no' => $this->terminal_no,
            ] + $params;
    }

    /**
     * Signature üretme methodu.
     *
     * @param $url string
     * @return string
     */
    private function generate_signature(string $url): string
    {
        $hashString = $this->endpoint . $url . $this->merchant_no . $this->terminal_no . $this->get_request_body_string();

        return hash_hmac('sha512', $hashString, $this->store_key);
    }

    /**
     * IFrame çağırma methodu.
     *
     * @return false|mixed
     * @throws HepsiPaySDKException
     */
    public function frame_init()
    {
        $signature = $this->generate_signature("v2/hepsipayframe/init");

        return $this->post("v2/hepsipayframe/init", $this->params([
            'signature' => $signature,
        ]));

    }

    /**
     * Kullanıcının HepsiPay hesabının olup olmadığını sorgulayan servis.
     *
     * @return false|mixed
     * @throws HepsiPaySDKException
     */
    public function check_availability()
    {
        $signature = $this->generate_signature("v2/hepsipayframe/checkavailability");

        return $this->post("v2/hepsipayframe/checkavailability", $this->params([
            'signature' => $signature,
        ]));

    }

    /**
     * Ödeme durumunu sorgulayan servis.
     *
     * @param string $token
     * @return false|mixed
     * @throws HepsiPaySDKException
     */
    public function payment_status(string $token)
    {

        $signature = $this->generate_signature("payment/" . $token . "/status");

        return $this->get("payment/" . $token . "/status", $this->params([
            'signature' => $signature,
        ]));

    }

    /**
     * Payment ID değerine göre iade işlemi yapan servis.
     *
     * @return false|mixed
     * @throws HepsiPaySDKException
     */
    public function payment_refund_with_payment_id()
    {

        $signature = $this->generate_signature("payment/refund");

        return $this->post("payment/refund", $this->params([
            'signature' => $signature,
        ]));

    }

    /**
     * Referencce No değerine göre iade işlemi yapan servis.
     *
     * @return false|mixed
     * @throws HepsiPaySDKException
     */
    public function payment_refund_with_reference_no()
    {

        $signature = $this->generate_signature("payment/refund");

        return $this->post("payment/refund", $this->params([
            'signature' => $signature,
        ]));

    }
}

/**
 *
 */
class HepsiPaySDKException extends \Exception
{
}

